/*
 * Emil
 * 05 August 2005 - 28 February 2008
 * 
 * ATmega128 code
*/

#define F_CPU 16000312L

#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include "ublox5.h"

volatile u08	*ftdi = ( u08 * ) 0x1100;	// external memory
char          disp[8];

#define I2C_SEND_DATA_BUFFER_SIZE 	BLEN
#define I2C_RECEIVE_DATA_BUFFER_SIZE 	BLEN

#include "i2c.c"

void
portsInit( void )
{
  SFIOR |= _BV( PUD );		// disable pull-ups for all ports

  XMCRB = 0x87;			// Bus keeper, full port C
//  XMCRA = _BV( SRW11 );       // extra delay
  MCUCR = _BV( SRE );		// enable external memory
//  MCUCR |= _BV( SRW10 );      // extra delay
}

inline        u08
ftdi_in( void )
{
  asm volatile  ( "nop\n\tnop" );	// minimum 2 cycles before testing

  while ( PINE & _BV( 5 ) ) ;
  return *ftdi;
}

inline void
ftdi_out( u08 c )
{
  asm volatile  ( "nop\n\tnop" );	// minimum 2 cycles before testing

  while ( PINE & _BV( 4 ) ) ;
  *ftdi = c;
}

int
dprintf( const char *fmt, ... )
{
  va_list       ap;
  u08           data[BLEN];
  int           n;

  va_start( ap, fmt );
  n = vsnprintf( ( char * ) data, BLEN, fmt, ap );
  va_end( ap );
  i2cMasterSend( 0x75, n, data );
  return n;
}

void
_delay_zs( int zs )
{
  zs *= 10;
  while ( zs-- )
    _delay_ms( 10 );
}

int
main( void )
{
  u08 reg = 0xfd;
  u08 gps[BLEN];
  u16 i, len, pkt;

  portsInit(  );
  i2cInit(  );
  _delay_zs( 1 );
  dprintf( "GPS-U5" );
  _delay_zs( 5 );

  memset( gps, 0, BLEN );
  while ( 1 )			// main loop
  {
    i2cMasterSend( 0x84, 1, &reg ); // set pointer to register 0xfd
    i2cMasterReceive( 0x84, 2, gps ); // get length of messages
    len = gps[0] << 8 |  gps[1];
    while( len )
    {
      pkt = len > BLEN ? BLEN : len;
      i2cMasterReceive( 0x84, pkt, gps ); // receive chunk of message
      for( i = 0; i < pkt; i++ )
        ftdi_out( gps[i] );
      len -= pkt;
    }
    _delay_zs( 1 );
  }
  return 0;
}
